package cn.com.qmth.examcloud.core.questions.base.json;

import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.nodes.TextNode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import freemarker.template.Configuration;
import freemarker.template.Template;

public class JsonExportUtil {

    private static final Logger log = LoggerFactory.getLogger(JsonExportUtil.class);

    public static final String QUESOPS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    public static final String ELEMENT_TYPE_RID = "imageId";

    public static final String ELEMENT_TYPE_TEXT = "text";

    public static final String ELEMENT_TYPE_IMG = "image";

    private static final String ENCODING = "UTF-8";

    private static Configuration config;

    private static Template questionSection;

    private static Template contentTypes;

    private static Template document;

    private static Template documentRel;

    private static Pattern heightRex = Pattern.compile("height:([\\s\\S]*?)px");

    private static Pattern widthRex = Pattern.compile("width:([\\s\\S]*?)px");
    static {

        config = new Configuration(Configuration.VERSION_2_3_25);
        // 设置编码
        config.setDefaultEncoding(ENCODING);
        // 设置ftl模板路径
        config.setClassForTemplateLoading(JsonExportUtil.class, "/export_template/origin_paper/");

        try {
            questionSection = config.getTemplate("question_section.ftl", ENCODING);
            contentTypes = config.getTemplate("content_types.ftl", ENCODING);
            document = config.getTemplate("document.ftl", ENCODING);
            documentRel = config.getTemplate("document_rel.ftl", ENCODING);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

    public static String getDocxBasePath() {
        return JsonExportUtil.class.getProtectionDomain().getCodeSource().getLocation().getPath()
                + "export_template/docx";
    }

    public static List<JSection> getSections(String html) {
        if (StringUtils.isBlank(html)) {
            return null;
        }
        html=html.trim();
        if (!html.startsWith("<p>")) {
            html = "<p>" + html + "</p>";
        }
        List<JSection> ss = new ArrayList<JSection>();
        Document doc = Jsoup.parse(html);
        Element b = doc.body();
        for (Node e : b.childNodes()) {
            if ("p".equals(e.nodeName()) && e.childNodeSize() > 0) {
                JSection s = new JSection();
                List<SectionElement> ses = new ArrayList<SectionElement>();
                s.setElements(ses);
                for (Node ce : e.childNodes()) {
                    getSectionElement(ce, ses);
                }
                ss.add(s);
            }
        }
        return ss;

    }

    private static void getSectionElement(Node ce, List<SectionElement> ses) {
        if ("span".equals(ce.nodeName()) && ce.childNodeSize() > 0) {
            for (Node e : ce.childNodes()) {
                getSectionElement(e, ses);
            }
        } else {
            SectionElement se = new SectionElement();
            if (ce instanceof TextNode) {
                TextNode tn = (TextNode) ce;
                se.setType(ELEMENT_TYPE_TEXT);
                se.setValue(tn.text());
            } else if (ce instanceof Element) {
                if ("img".equals(ce.nodeName())) {
                    se.setType(ELEMENT_TYPE_IMG);
                    se.setValue(ce.attr("src"));
                    SectionElementParams sep = new SectionElementParams();
                    sep.setHeight(getHeight(ce));
                    sep.setWidth(getWidth(ce));
                    se.setParams(sep);
                } else {
                    Element el = (Element) ce;
                    se.setType(ELEMENT_TYPE_TEXT);
                    se.setValue(el.text());
                }
            }
            ses.add(se);
        }
    }

    private static int getWidth(Node ce) {
        int n = 50;
        String str = ce.attr("width");
        if (StringUtils.isNoneBlank(str)) {
            return getWordImgSize(str);
        }
        String style = ce.attr("style");
        if (StringUtils.isNoneBlank(style)) {
            Matcher m = widthRex.matcher(style);
            if (m.find()) {
                return Integer.valueOf(m.group(1)) * 9525;
            }
        }
        return n;
    }

    private static int getHeight(Node ce) {
        int n = 50;
        String str = ce.attr("height");
        if (StringUtils.isNoneBlank(str)) {
            return getWordImgSize(str);
        }
        String style = ce.attr("style");
        if (StringUtils.isNoneBlank(style)) {
            Matcher m = heightRex.matcher(style);
            if (m.find()) {
                return Integer.valueOf(m.group(1)) * 9525;
            }
        }
        return n;
    }

    private static Integer getWordImgSize(String px) {
        if (StringUtils.isBlank(px)) {
            return null;
        }
        String sizeStr = px.replace("px", "");
        Integer size = Integer.valueOf(sizeStr) * 9525;
        return size;
    }

    public static String getDocumentDoc(Object paperExp) {
        StringWriter result = null;
        try {
            result = new StringWriter();
            document.process(paperExp, result);
            return result.toString();
        } catch (Exception e) {
            throw new ExamCloudRuntimeException(e);
        }
    }

    public static String getDocumentRelDoc(Map<String, Object> map) {
        StringWriter result = null;
        try {
            result = new StringWriter();
            documentRel.process(map, result);
            return result.toString();
        } catch (Exception e) {
            throw new ExamCloudRuntimeException(e);
        }
    }

    public static String getQuestionSectionDoc(Map<String, Object> map) {
        StringWriter result = null;
        try {
            result = new StringWriter();
            questionSection.process(map, result);
            return result.toString();
        } catch (Exception e) {
            throw new ExamCloudRuntimeException(e);
        }
    }

    public static String getContentTypesDoc(Map<String, Object> map) {
        StringWriter result = null;
        try {
            result = new StringWriter();
            contentTypes.process(map, result);
            return result.toString();
        } catch (Exception e) {
            throw new ExamCloudRuntimeException(e);
        }
    }

}