package cn.com.qmth.examcloud.core.questions.service.impl;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.core.questions.base.FileDisposeUtil;
import cn.com.qmth.examcloud.core.questions.base.IdUtils;
import cn.com.qmth.examcloud.core.questions.base.dto.ExportTemplateDto;
import cn.com.qmth.examcloud.core.questions.base.enums.ExportTemplateType;
import cn.com.qmth.examcloud.core.questions.dao.ExportTemplateRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.ExportTemplateEntity;
import cn.com.qmth.examcloud.core.questions.service.ExportTemplateService;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.upyun.UpyunPathEnvironmentInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunService;

@Service("exportTemplateService")
public class ExportTemplateServiceImpl implements ExportTemplateService {

    protected ExamCloudLog log = ExamCloudLogFactory.getLog(this.getClass());

    @Autowired
    private ExportTemplateRepo exportTemplateRepo;

    @Autowired
    private UpyunService upyunService;

    @Override
    public Page<ExportTemplateDto> getPage(Long rootOrgId, String fileName, String type, int curPage, int pageSize) {
        Specification<ExportTemplateEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();

            predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));

            if (StringUtils.isNotBlank(fileName)) {
                predicates.add(cb.like(root.get("fileName"), toSqlSearchPattern(fileName)));
            }
            if (StringUtils.isNotBlank(type)) {
                predicates.add(cb.like(root.get("type"), toSqlSearchPattern(type)));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        PageRequest pageRequest = PageRequest.of(curPage - 1, pageSize, Sort.by(Direction.DESC, "creationTime"));

        Page<ExportTemplateEntity> page = exportTemplateRepo.findAll(specification, pageRequest);
        List<ExportTemplateEntity> list = page.getContent();
        List<ExportTemplateDto> retList = new ArrayList<ExportTemplateDto>();
        if (list != null && list.size() > 0) {
            String upyunFileUrl = PropertyHolder.getString("$upyun.site.1.domain");
            for (ExportTemplateEntity e : list) {
                ExportTemplateDto dto = new ExportTemplateDto();
                BeanUtils.copyProperties(e, dto);
                dto.setFullFilePath(upyunFileUrl + dto.getFilePath());
                dto.setTypeName(ExportTemplateType.getName(dto.getType()));
                retList.add(dto);
            }
        }
        return new PageImpl<ExportTemplateDto>(retList, PageRequest.of(curPage - 1, pageSize), page.getTotalElements());
    }

    @Transactional
    @Override
    public void addFile(User user, String templateName, String type, MultipartFile dataFile) {
        Long rootOrgId = user.getRootOrgId();
        if (StringUtils.isBlank(templateName)) {
            throw new StatusException("10001", "模板名称不能为空");
        }
        if (!ExportTemplateType.checkCode(type)) {
            throw new StatusException("10002", "模板类型错误");
        }
        ExportTemplateEntity e = new ExportTemplateEntity();
        String fName = dataFile.getOriginalFilename();
        e.setFileName(templateName);
        e.setType(type);
        e.setEnable(false);
        e.setCreationTime(new Date());
        e.setCreateUser(user.getDisplayName());
        e.setFileKey(IdUtils.uuid());
        e.setRootOrgId(rootOrgId);
        String suffix = fName.substring(fName.lastIndexOf(".") + 1, fName.length());
        e.setSuffix(suffix);

        String relativePath = e.getFileKey() + "." + e.getSuffix();
        UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
        env.setRootOrgId(rootOrgId.toString());
        env.setRelativePath(relativePath);
        String fpath = null;
        InputStream is = null;
        try {
            is = dataFile.getInputStream();
            fpath = upyunService.writeFile("export_template", env, is, null).getRelativePath();
        } catch (IOException e1) {
            throw new StatusException("10003", "模板上传出错");
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                    log.warn("InputStream close faild!");
                }
            }
        }
        e.setFilePath(fpath);
        exportTemplateRepo.save(e);
    }

    @Transactional
    @Override
    public void delete(Long rootOrgId, Long id) {
        exportTemplateRepo.deleteByRootOrgIdAndId(rootOrgId, id);
    }

    @Transactional
    @Override
    public void enable(Long rootOrgId, Long id) {
        ExportTemplateEntity et = exportTemplateRepo.findByIdAndRootOrgId(id, rootOrgId);
        if (et != null) {
            exportTemplateRepo.updateEnableByType(rootOrgId, et.getType(), false);
            exportTemplateRepo.updateEnableById(rootOrgId, id, true);
        } else {
            throw new StatusException("30001", "模板不存在");
        }

    }

    @Transactional
    @Override
    public void disenable(Long rootOrgId, Long id) {
        exportTemplateRepo.updateEnableById(rootOrgId, id, false);
    }

    @Override
    public String getExportTemplateName(ExportTemplateType type, Long rootOrgId) {
        String temPath = PropertyHolder.getString("examcloud.web.sys.tempDataDir");
        List<ExportTemplateEntity> list = exportTemplateRepo.findByRootOrgIdAndTypeAndEnable(rootOrgId, type.getCode(),
                true);
        if (list == null || list.size() == 0) {
            throw new StatusException("50001", "模板未设置或未启用");
        }
        if (list != null && list.size() > 1) {
            throw new StatusException("50002", "存在多个启用的模板");
        }
        ExportTemplateEntity et = list.get(0);
        String fileName = et.getFileKey() + "." + et.getSuffix();
        String filePath = temPath + "/" + rootOrgId + "/" + fileName;
        File f = new File(filePath);
        if (!f.exists()) {
            String upyunFileUrl = PropertyHolder.getString("$upyun.site.1.domain");
            createExportTemplateFile(upyunFileUrl + et.getFilePath(), filePath);
        }
        return fileName;
    }

    private synchronized void createExportTemplateFile(String fileUrl, String localFilePath) {
        File f = new File(localFilePath);
        File dir = f.getParentFile();
        try {
            if (!dir.exists()) {
                dir.mkdirs();
            }
            f.createNewFile();
        } catch (IOException e) {
            throw new StatusException("60001", "创建模板文件失败");
        }
        if (!FileDisposeUtil.saveUrlAs(fileUrl, localFilePath)) {
            throw new StatusException("60002", "获取模板到本地失败");
        }
    }

    private String toSqlSearchPattern(String column) {
        if (StringUtils.isBlank(column)) {
            return "%";
        } else {
            column = column.trim().replaceAll("\\s", "%");
            column = "%" + column + "%";
            return column;
        }
    }

}